// Include the most common headers from the C standard library
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

// Include the main libnx system header, for Switch development
#include <switch.h>

size_t __nx_heap_size = 0x200000;
u32 __nx_applet_type = AppletType_LibraryApplet;

// See also libnx web.h.

SwkbdTextCheckResult validate_text(char* tmp_string, size_t tmp_string_size) {
    if (strncmp(tmp_string, "http", 4)) {
        strncpy(tmp_string, "Please enter a valid URL", tmp_string_size);
        return SwkbdTextCheckResult_Bad;
    }

    return SwkbdTextCheckResult_OK;
}

// Main program entrypoint
int main(int argc, char* argv[])
{
    Result rc=0;

    SwkbdConfig kbd;
    char url[512] = { "https://google.com/" };
    rc = swkbdCreate(&kbd, 0);
    if (R_SUCCEEDED(rc)) {
        swkbdConfigMakePresetDefault(&kbd);
        swkbdConfigSetOkButtonText(&kbd, "Go");
        swkbdConfigSetGuideText(&kbd, "http(s)://");
        swkbdConfigSetTextCheckCallback(&kbd, validate_text);
    }

    // Main loop
    while (appletMainLoop())
    {
        WebCommonConfig config;
        WebCommonReply reply;
		WebExitReason exitReason = WebExitReason_ExitButton;

        // Create the config. There's a number of web*Create() funcs, see libnx web.h.
        // webPageCreate/webNewsCreate requires running under a host title which has HtmlDocument content, when the title is an Application. When the title is an Application when using webPageCreate/webNewsCreate, and webConfigSetWhitelist is not used, the whitelist will be loaded from the content. Atmosphère hbl_html can be used to handle this.
        rc = webPageCreate(&config, url);

        if (R_SUCCEEDED(rc)) {
            // At this point you can use any webConfigSet* funcs you want.

            rc = webConfigSetWhitelist(&config, "^http*"); // Set the whitelist, adjust as needed. If you're only using a single domain, you could remove this and use webNewsCreate for the above (see web.h for webNewsCreate).

            if (R_SUCCEEDED(rc)) { // Launch the applet and wait for it to exit.
                rc = webConfigShow(&config, &reply); // If you don't use reply you can pass NULL for it.
            }

			if (R_SUCCEEDED(rc)) { // Normally you can ignore exitReason.
				rc = webReplyGetExitReason(&reply, &exitReason);
			}
        }

		if (R_SUCCEEDED(rc) && exitReason == WebExitReason_ExitButton) {
        	swkbdConfigSetInitialText(&kbd, url);
	        rc = swkbdShow(&kbd, url, sizeof(url));
	        if (R_FAILED(rc)) {
	            break;
	        }
		} else {
			break;
		}
    }

    swkbdClose(&kbd);
    return 0;
}
